import { ListenerRssAggregator } from 'listener-rss-agregator';
import { ListenerRSSInfos } from 'listener-rss';
type ListenerData = {
    ChannelId: number;
    firstUpdate: boolean;
} & ListenerRSSInfos;

let myManager: ListenerRssAggregator;
let listenersDataBinding = new Map<string, ListenerData>();
let logger: any;
let firstUpdate = true;

import * as path from 'path';
import fs from 'fs';

async function register({
    registerSetting,
    settingsManager,
    peertubeHelpers,
}: any) {
    const basePath = peertubeHelpers.plugin.getDataDirectoryPath();
    logger = peertubeHelpers.logger;

    registerSetting({
        name: 'ytb-urls',
        label: 'liste des urls youtube a auto-importer',
        type: 'input-textarea',
    });

    logger.warn('setting register');
    fs.appendFileSync(path.join(basePath, '/storage.bd'), ''); // append

    const configAggregator = await ListenerRssAggregator.instantiateAggregator(
        path.join(basePath, '/storage.bd')
    );
    myManager = new ListenerRssAggregator(configAggregator);

    logger.warn('Aggregator created');

    const inputs = await settingsManager.getSetting('ytb-urls');
    if (inputs) await addListeners(inputs);

    logger.warn('Config loaded');

    settingsManager.onSettingsChange(async (settings: any) => {
        await addListeners(settings['ytb-urls']);
    });

    myManager.on('newEntries', (entries: any) => {
        if (firstUpdate) {
            logger.warn('Premiere update');

            firstUpdate = false;
            return;
        }

        logger.warn(
            'Nouvelles baba entrées détéctées: ' + JSON.stringify(entries)
        );
    });
}

async function addListeners(listenerInput: string) {
    let objs: ListenerData[];
    try {
        objs = JSON.parse(listenerInput);
    } catch {
        logger.warn('Erreur: malformé');
        return;
    }

    for (const item of objs) {
        if (listenersDataBinding.has(item.address)) break;
        listenersDataBinding.set(item.address, {
            ChannelId: item.ChannelId,
            firstUpdate: item.firstUpdate,
            address: item.address,
        });
    }

    myManager.stopAll();
    await myManager.saveOverride(objs);
    firstUpdate = true;

    if (logger) logger.warn('Configuration modifiée: ' + listenerInput);

    myManager.startAll();
}

async function unregister() {
    myManager.stopAll();
    return;
}

module.exports = {
    register,
    unregister,
};
